"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.main = void 0;
var fs_1 = __importDefault(require("fs"));
var path_1 = __importDefault(require("path"));
var kleur_1 = __importDefault(require("kleur"));
var pretty_print_error_1 = require("pretty-print-error");
// @ts-ignore
var package_json_1 = __importDefault(require("../package.json"));
function printUsage() {
    console.log("suchibot ".concat(package_json_1.default.version));
    console.log("");
    console.log("Usage:");
    console.log("  suchibot [path to script file]");
    console.log("");
    console.log("If no script file path is specified, the program will attempt to load script.ts or script.js (whichever exists).");
    console.log("");
    console.log("Examples:");
    console.log("  suchibot ./my-script.js");
}
function main(suchibot) {
    if (process.argv.includes("-h") || process.argv.includes("--help")) {
        printUsage();
        process.exit(0);
    }
    if (process.argv.includes("-v") || process.argv.includes("--version")) {
        console.log("".concat(package_json_1.default.version));
        process.exit(0);
    }
    var modulePath = process.argv[2];
    var defaultTsPath = path_1.default.resolve(process.cwd(), "./script.ts");
    var defaultJsPath = path_1.default.resolve(process.cwd(), "./script.js");
    if (modulePath == null) {
        if (fs_1.default.existsSync(defaultTsPath)) {
            modulePath = defaultTsPath;
        }
        else if (fs_1.default.existsSync(defaultJsPath)) {
            modulePath = defaultJsPath;
        }
        else {
            console.log(kleur_1.default.red("Not sure which file to run. Please either specify a script as the first argument to suchibot, or create a file named ".concat(kleur_1.default.yellow("script.ts"), " or ").concat(kleur_1.default.yellow("script.js"), " in the current folder.")));
            console.log("");
            printUsage();
            process.exit(1);
        }
    }
    if (!path_1.default.isAbsolute(modulePath)) {
        modulePath = path_1.default.resolve(process.cwd(), modulePath);
    }
    if (!fs_1.default.existsSync(modulePath)) {
        console.log("Trying to load non-existent file:", modulePath);
        console.log("");
        printUsage();
        process.exit(1);
    }
    suchibot.startListening();
    console.log(kleur_1.default.green("Now listening for mouse/keyboard events. Press Ctrl+C to exit at any time."));
    process.on("unhandledRejection", function (error) {
        console.log(kleur_1.default.red("An unhandled Promise rejection occurred:"));
        console.log((0, pretty_print_error_1.formatError)(error));
    });
    try {
        require(modulePath);
    }
    catch (err) {
        console.log(kleur_1.default.red("An error occurred in your script:"));
        console.log((0, pretty_print_error_1.formatError)(err));
        suchibot.stopListening();
        process.exit(1);
    }
}
exports.main = main;
