"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var _a;
Object.defineProperty(exports, "__esModule", { value: true });
exports.Keyboard = exports.KeyboardEvent = exports.isKeyboardEvent = void 0;
var types_1 = require("../types");
var uiohook_napi_1 = require("uiohook-napi");
var mitt_1 = __importDefault(require("mitt"));
var held_keys_1 = require("./held-keys");
var held_mouse_buttons_1 = require("./held-mouse-buttons");
// ------------ keyboard stuff -------------
var IS_KEYBOARD_EVENT = Symbol("IS_KEYBOARD_EVENT");
function isKeyboardEvent(event) {
    return typeof event === "object" && event != null && event[IS_KEYBOARD_EVENT];
}
exports.isKeyboardEvent = isKeyboardEvent;
var KeyboardEvent = /** @class */ (function () {
    function KeyboardEvent(type, key, modifierKeys, mouseButtons) {
        if (modifierKeys === void 0) { modifierKeys = (0, held_keys_1.getModifierKeysState)(); }
        if (mouseButtons === void 0) { mouseButtons = (0, held_mouse_buttons_1.getMouseButtonsState)(); }
        this.type = type;
        this.key = key;
        this.modifierKeys = modifierKeys;
        this.mouseButtons = mouseButtons;
        this[IS_KEYBOARD_EVENT] = true;
    }
    return KeyboardEvent;
}());
exports.KeyboardEvent = KeyboardEvent;
var uioToKeyMap = (_a = {},
    _a[uiohook_napi_1.UiohookKey.Tab] = types_1.Key.TAB,
    _a[uiohook_napi_1.UiohookKey.Enter] = types_1.Key.ENTER,
    _a[uiohook_napi_1.UiohookKey.CapsLock] = types_1.Key.CAPS_LOCK,
    _a[uiohook_napi_1.UiohookKey.Escape] = types_1.Key.ESCAPE,
    _a[uiohook_napi_1.UiohookKey.Space] = types_1.Key.SPACE,
    _a[uiohook_napi_1.UiohookKey.PageUp] = types_1.Key.PAGE_UP,
    _a[uiohook_napi_1.UiohookKey.PageDown] = types_1.Key.PAGE_DOWN,
    _a[uiohook_napi_1.UiohookKey.End] = types_1.Key.END,
    _a[uiohook_napi_1.UiohookKey.Home] = types_1.Key.HOME,
    _a[uiohook_napi_1.UiohookKey.ArrowLeft] = types_1.Key.LEFT,
    _a[uiohook_napi_1.UiohookKey.ArrowUp] = types_1.Key.UP,
    _a[uiohook_napi_1.UiohookKey.ArrowRight] = types_1.Key.RIGHT,
    _a[uiohook_napi_1.UiohookKey.ArrowDown] = types_1.Key.DOWN,
    _a[uiohook_napi_1.UiohookKey.Insert] = types_1.Key.INSERT,
    _a[uiohook_napi_1.UiohookKey.Delete] = types_1.Key.DELETE,
    _a[uiohook_napi_1.UiohookKey[0]] = types_1.Key.ZERO,
    _a[uiohook_napi_1.UiohookKey[1]] = types_1.Key.ONE,
    _a[uiohook_napi_1.UiohookKey[2]] = types_1.Key.TWO,
    _a[uiohook_napi_1.UiohookKey[3]] = types_1.Key.THREE,
    _a[uiohook_napi_1.UiohookKey[4]] = types_1.Key.FOUR,
    _a[uiohook_napi_1.UiohookKey[5]] = types_1.Key.FIVE,
    _a[uiohook_napi_1.UiohookKey[6]] = types_1.Key.SIX,
    _a[uiohook_napi_1.UiohookKey[7]] = types_1.Key.SEVEN,
    _a[uiohook_napi_1.UiohookKey[8]] = types_1.Key.EIGHT,
    _a[uiohook_napi_1.UiohookKey[9]] = types_1.Key.NINE,
    _a[uiohook_napi_1.UiohookKey.A] = types_1.Key.A,
    _a[uiohook_napi_1.UiohookKey.B] = types_1.Key.B,
    _a[uiohook_napi_1.UiohookKey.C] = types_1.Key.C,
    _a[uiohook_napi_1.UiohookKey.D] = types_1.Key.D,
    _a[uiohook_napi_1.UiohookKey.E] = types_1.Key.E,
    _a[uiohook_napi_1.UiohookKey.F] = types_1.Key.F,
    _a[uiohook_napi_1.UiohookKey.G] = types_1.Key.G,
    _a[uiohook_napi_1.UiohookKey.H] = types_1.Key.H,
    _a[uiohook_napi_1.UiohookKey.I] = types_1.Key.I,
    _a[uiohook_napi_1.UiohookKey.J] = types_1.Key.J,
    _a[uiohook_napi_1.UiohookKey.K] = types_1.Key.K,
    _a[uiohook_napi_1.UiohookKey.L] = types_1.Key.L,
    _a[uiohook_napi_1.UiohookKey.M] = types_1.Key.M,
    _a[uiohook_napi_1.UiohookKey.N] = types_1.Key.N,
    _a[uiohook_napi_1.UiohookKey.O] = types_1.Key.O,
    _a[uiohook_napi_1.UiohookKey.P] = types_1.Key.P,
    _a[uiohook_napi_1.UiohookKey.Q] = types_1.Key.Q,
    _a[uiohook_napi_1.UiohookKey.R] = types_1.Key.R,
    _a[uiohook_napi_1.UiohookKey.S] = types_1.Key.S,
    _a[uiohook_napi_1.UiohookKey.T] = types_1.Key.T,
    _a[uiohook_napi_1.UiohookKey.U] = types_1.Key.U,
    _a[uiohook_napi_1.UiohookKey.V] = types_1.Key.V,
    _a[uiohook_napi_1.UiohookKey.W] = types_1.Key.W,
    _a[uiohook_napi_1.UiohookKey.X] = types_1.Key.X,
    _a[uiohook_napi_1.UiohookKey.Y] = types_1.Key.Y,
    _a[uiohook_napi_1.UiohookKey.Z] = types_1.Key.Z,
    _a[uiohook_napi_1.UiohookKey.Numpad0] = types_1.Key.NUMPAD_0,
    _a[uiohook_napi_1.UiohookKey.Numpad1] = types_1.Key.NUMPAD_1,
    _a[uiohook_napi_1.UiohookKey.Numpad2] = types_1.Key.NUMPAD_2,
    _a[uiohook_napi_1.UiohookKey.Numpad3] = types_1.Key.NUMPAD_3,
    _a[uiohook_napi_1.UiohookKey.Numpad4] = types_1.Key.NUMPAD_4,
    _a[uiohook_napi_1.UiohookKey.Numpad5] = types_1.Key.NUMPAD_5,
    _a[uiohook_napi_1.UiohookKey.Numpad6] = types_1.Key.NUMPAD_6,
    _a[uiohook_napi_1.UiohookKey.Numpad7] = types_1.Key.NUMPAD_7,
    _a[uiohook_napi_1.UiohookKey.Numpad8] = types_1.Key.NUMPAD_8,
    _a[uiohook_napi_1.UiohookKey.Numpad9] = types_1.Key.NUMPAD_9,
    _a[uiohook_napi_1.UiohookKey.NumpadMultiply] = types_1.Key.NUMPAD_MULTIPLY,
    _a[uiohook_napi_1.UiohookKey.NumpadAdd] = types_1.Key.NUMPAD_ADD,
    _a[uiohook_napi_1.UiohookKey.NumpadSubtract] = types_1.Key.NUMPAD_SUBTRACT,
    _a[uiohook_napi_1.UiohookKey.NumpadDecimal] = types_1.Key.NUMPAD_DECIMAL,
    _a[3612] = types_1.Key.NUMPAD_ENTER,
    _a[uiohook_napi_1.UiohookKey.NumpadDivide] = types_1.Key.NUMPAD_DIVIDE,
    _a[uiohook_napi_1.UiohookKey.F1] = types_1.Key.F1,
    _a[uiohook_napi_1.UiohookKey.F2] = types_1.Key.F2,
    _a[uiohook_napi_1.UiohookKey.F3] = types_1.Key.F3,
    _a[uiohook_napi_1.UiohookKey.F4] = types_1.Key.F4,
    _a[uiohook_napi_1.UiohookKey.F5] = types_1.Key.F5,
    _a[uiohook_napi_1.UiohookKey.F6] = types_1.Key.F6,
    _a[uiohook_napi_1.UiohookKey.F7] = types_1.Key.F7,
    _a[uiohook_napi_1.UiohookKey.F8] = types_1.Key.F8,
    _a[uiohook_napi_1.UiohookKey.F9] = types_1.Key.F9,
    _a[uiohook_napi_1.UiohookKey.F10] = types_1.Key.F10,
    _a[uiohook_napi_1.UiohookKey.F11] = types_1.Key.F11,
    _a[uiohook_napi_1.UiohookKey.F12] = types_1.Key.F12,
    _a[uiohook_napi_1.UiohookKey.F13] = types_1.Key.F13,
    _a[uiohook_napi_1.UiohookKey.F14] = types_1.Key.F14,
    _a[uiohook_napi_1.UiohookKey.F15] = types_1.Key.F15,
    _a[uiohook_napi_1.UiohookKey.F16] = types_1.Key.F16,
    _a[uiohook_napi_1.UiohookKey.F17] = types_1.Key.F17,
    _a[uiohook_napi_1.UiohookKey.F18] = types_1.Key.F18,
    _a[uiohook_napi_1.UiohookKey.F19] = types_1.Key.F19,
    _a[uiohook_napi_1.UiohookKey.F20] = types_1.Key.F20,
    _a[uiohook_napi_1.UiohookKey.F21] = types_1.Key.F21,
    _a[uiohook_napi_1.UiohookKey.F22] = types_1.Key.F22,
    _a[uiohook_napi_1.UiohookKey.F23] = types_1.Key.F23,
    _a[uiohook_napi_1.UiohookKey.F24] = types_1.Key.F24,
    _a[uiohook_napi_1.UiohookKey.Semicolon] = types_1.Key.SEMICOLON,
    _a[uiohook_napi_1.UiohookKey.Equal] = types_1.Key.EQUAL,
    _a[uiohook_napi_1.UiohookKey.Comma] = types_1.Key.COMMA,
    _a[uiohook_napi_1.UiohookKey.Minus] = types_1.Key.MINUS,
    _a[uiohook_napi_1.UiohookKey.Period] = types_1.Key.PERIOD,
    _a[uiohook_napi_1.UiohookKey.Slash] = types_1.Key.SLASH,
    _a[uiohook_napi_1.UiohookKey.Backquote] = types_1.Key.BACKTICK,
    _a[uiohook_napi_1.UiohookKey.BracketLeft] = types_1.Key.LEFT_BRACKET,
    _a[uiohook_napi_1.UiohookKey.Backslash] = types_1.Key.BACKSLASH,
    _a[uiohook_napi_1.UiohookKey.BracketRight] = types_1.Key.RIGHT_BRACKET,
    _a[uiohook_napi_1.UiohookKey.Quote] = types_1.Key.QUOTE,
    _a[uiohook_napi_1.UiohookKey.Ctrl] = types_1.Key.LEFT_CONTROL,
    _a[uiohook_napi_1.UiohookKey.Alt] = types_1.Key.LEFT_ALT,
    _a[uiohook_napi_1.UiohookKey.Shift] = types_1.Key.LEFT_SHIFT,
    _a[70] = types_1.Key.SCROLL_LOCK,
    _a[3653] = types_1.Key.PAUSE_BREAK,
    _a[14] = types_1.Key.BACKSPACE,
    _a[69] = types_1.Key.NUM_LOCK,
    _a[54] = types_1.Key.RIGHT_SHIFT,
    _a[3613] = types_1.Key.RIGHT_CONTROL,
    _a[3640] = types_1.Key.RIGHT_ALT,
    _a[3675] = types_1.Key.LEFT_SUPER,
    _a[3676] = types_1.Key.RIGHT_SUPER,
    _a[57376] = types_1.Key.MUTE,
    _a[57390] = types_1.Key.VOLUME_DOWN,
    _a[57392] = types_1.Key.VOLUME_UP,
    _a);
function uioToKey(uioKey) {
    return uioToKeyMap[uioKey] || null;
}
// ------------ event emitter setup -------------
var events = (0, mitt_1.default)();
uiohook_napi_1.uIOhook.on("keydown", function (event) {
    var key = uioToKey(event.keycode);
    if (!key) {
        console.warn("WARNING: received keydown for unsupported keycode:", event.keycode);
        return;
    }
    (0, held_keys_1.setKeyState)(key, "down");
    var newEvent = new KeyboardEvent("down", key);
    events.emit("keydown", newEvent);
});
uiohook_napi_1.uIOhook.on("keyup", function (event) {
    var key = uioToKey(event.keycode);
    if (!key) {
        console.warn("WARNING: received keyup for unsupported keycode:", event.keycode);
        return;
    }
    (0, held_keys_1.setKeyState)(key, "up");
    var newEvent = new KeyboardEvent("up", key);
    events.emit("keyup", newEvent);
});
// ------------ public API -------------
exports.Keyboard = {
    onDown: function (key, eventHandler) {
        var callback = function (event) {
            if (key === types_1.Key.ANY || event.key === key) {
                eventHandler(event);
            }
        };
        events.on("keydown", callback);
        return {
            stop: function () {
                events.off("keydown", callback);
            },
        };
    },
    onUp: function (key, eventHandler) {
        var callback = function (event) {
            if (key === types_1.Key.ANY || event.key === key) {
                eventHandler(event);
            }
        };
        events.on("keyup", callback);
        return {
            stop: function () {
                events.off("keyup", callback);
            },
        };
    },
    isDown: function (key) {
        return (0, held_keys_1.isKeyDown)(key);
    },
    isUp: function (key) {
        return (0, held_keys_1.isKeyUp)(key);
    },
};
