"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Mouse = exports.MouseEvent = exports.isMouseEvent = void 0;
var types_1 = require("../types");
var uiohook_napi_1 = require("uiohook-napi");
var mitt_1 = __importDefault(require("mitt"));
var held_keys_1 = require("./held-keys");
var held_mouse_buttons_1 = require("./held-mouse-buttons");
// ------------ mouse stuff -------------
var IS_MOUSE_EVENT = Symbol("IS_MOUSE_EVENT");
function isMouseEvent(event) {
    return typeof event === "object" && event != null && event[IS_MOUSE_EVENT];
}
exports.isMouseEvent = isMouseEvent;
var MouseEvent = /** @class */ (function () {
    function MouseEvent(type, button, x, y, modifierKeys, mouseButtons) {
        if (modifierKeys === void 0) { modifierKeys = (0, held_keys_1.getModifierKeysState)(); }
        if (mouseButtons === void 0) { mouseButtons = (0, held_mouse_buttons_1.getMouseButtonsState)(); }
        this.type = type;
        this.button = button;
        this.x = x;
        this.y = y;
        this.modifierKeys = modifierKeys;
        this.mouseButtons = mouseButtons;
        this[IS_MOUSE_EVENT] = true;
    }
    return MouseEvent;
}());
exports.MouseEvent = MouseEvent;
var uioToMouseButtonMap = {
    1: types_1.MouseButton.LEFT,
    2: types_1.MouseButton.MIDDLE,
    3: types_1.MouseButton.RIGHT,
    4: types_1.MouseButton.MOUSE4,
    5: types_1.MouseButton.MOUSE5,
};
function uioToMouseButton(uioButton) {
    return uioToMouseButtonMap[uioButton] || null;
}
// ------------ event emitter setup -------------
var events = (0, mitt_1.default)();
uiohook_napi_1.uIOhook.on("click", function (event) {
    var button = uioToMouseButton(event.button);
    if (!button) {
        console.warn("WARNING: received click for unsupported mouse button:", event.button);
        return;
    }
    (0, held_mouse_buttons_1.setButtonState)(button, "up");
    var newEvent = new MouseEvent("click", button, event.x, event.y);
    events.emit("click", newEvent);
});
uiohook_napi_1.uIOhook.on("mousedown", function (event) {
    var button = uioToMouseButton(event.button);
    if (!button) {
        console.warn("WARNING: received mousedown for unsupported mouse button:", event.button);
        return;
    }
    (0, held_mouse_buttons_1.setButtonState)(button, "down");
    var newEvent = new MouseEvent("down", button, event.x, event.y);
    events.emit("mousedown", newEvent);
});
uiohook_napi_1.uIOhook.on("mouseup", function (event) {
    var button = uioToMouseButton(event.button);
    if (!button) {
        console.warn("WARNING: received mouseup for unsupported mouse button:", event.button);
        return;
    }
    (0, held_mouse_buttons_1.setButtonState)(button, "up");
    var newEvent = new MouseEvent("up", button, event.x, event.y);
    events.emit("mouseup", newEvent);
});
uiohook_napi_1.uIOhook.on("mousemove", function (event) {
    var newEvent = new MouseEvent("move", null, event.x, event.y);
    events.emit("mousemove", newEvent);
});
// ------------ public API -------------
exports.Mouse = {
    onDown: function (button, eventHandler) {
        var callback = function (event) {
            if (String(event.button) === String(button) ||
                String(button) === String(types_1.MouseButton.ANY)) {
                eventHandler(event);
            }
        };
        events.on("mousedown", callback);
        return {
            stop: function () {
                events.off("mousedown", callback);
            },
        };
    },
    onUp: function (button, eventHandler) {
        var callback = function (event) {
            if (String(event.button) === String(button) ||
                String(button) === String(types_1.MouseButton.ANY)) {
                eventHandler(event);
            }
        };
        events.on("mouseup", callback);
        return {
            stop: function () {
                events.off("mouseup", callback);
            },
        };
    },
    onClick: function (button, eventHandler) {
        var callback = function (event) {
            if (String(event.button) === String(button) ||
                String(button) === String(types_1.MouseButton.ANY)) {
                eventHandler(event);
            }
        };
        events.on("click", callback);
        return {
            stop: function () {
                events.off("click", callback);
            },
        };
    },
    onMove: function (eventHandler) {
        var callback = function (event) {
            eventHandler(event);
        };
        events.on("mousemove", callback);
        return {
            stop: function () {
                events.off("mousemove", callback);
            },
        };
    },
    isDown: function (button) {
        return (0, held_mouse_buttons_1.isButtonDown)(button);
    },
    isUp: function (button) {
        return (0, held_mouse_buttons_1.isButtonUp)(button);
    },
};
