"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.eventMatchesFilter = exports.keyboardEventFilter = exports.mouseEventFilter = void 0;
var input_1 = require("../input");
var types_1 = require("../types");
function mouseEventFilter(criteria) {
    if (criteria === void 0) { criteria = {}; }
    return __assign({ filterType: "Mouse" }, criteria);
}
exports.mouseEventFilter = mouseEventFilter;
function keyboardEventFilter(criteria) {
    if (criteria === void 0) { criteria = {}; }
    return __assign({ filterType: "Keyboard" }, criteria);
}
exports.keyboardEventFilter = keyboardEventFilter;
function mouseEventMatchesFilter(event, filter) {
    var doesMatch = true;
    if (filter.type != null) {
        doesMatch = doesMatch && filter.type === event.type;
    }
    if (filter.button != null && filter.button !== types_1.MouseButton.ANY) {
        doesMatch = doesMatch && filter.button === event.button;
    }
    if (filter.x != null) {
        doesMatch = doesMatch && filter.x === event.x;
    }
    if (filter.y != null) {
        doesMatch = doesMatch && filter.y === event.y;
    }
    return doesMatch;
}
function keyboardEventMatchesFilter(event, filter) {
    var doesMatch = true;
    if (filter.type != null) {
        doesMatch = doesMatch && filter.type === event.type;
    }
    if (filter.key != null && filter.key !== types_1.Key.ANY) {
        doesMatch = doesMatch && filter.key === event.key;
    }
    if (doesMatch && filter.modifierKeys != null) {
        for (var key in filter.modifierKeys) {
            if (Object.prototype.hasOwnProperty.call(filter.modifierKeys, key)) {
                doesMatch =
                    doesMatch && event.modifierKeys[key] === filter.modifierKeys[key];
                if (!doesMatch) {
                    break;
                }
            }
        }
    }
    return doesMatch;
}
function eventMatchesFilter(event, filter) {
    if ((0, input_1.isMouseEvent)(event) && filter.filterType === "Mouse") {
        return mouseEventMatchesFilter(event, filter);
    }
    else if ((0, input_1.isKeyboardEvent)(event) && filter.filterType === "Keyboard") {
        return keyboardEventMatchesFilter(event, filter);
    }
    else {
        return false;
    }
}
exports.eventMatchesFilter = eventMatchesFilter;
