"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TapeData = void 0;
var input_1 = require("../input");
var TapeData = /** @class */ (function () {
    function TapeData(actions) {
        this.actions = actions;
    }
    Object.defineProperty(TapeData.prototype, "length", {
        get: function () {
            var actions = this.actions;
            if (actions.length > 0) {
                var lastAction = actions[actions.length - 1];
                return lastAction.time;
            }
            else {
                return 0;
            }
        },
        enumerable: false,
        configurable: true
    });
    TapeData.deserialize = function (data) {
        var actions = data.actions.map(function (serializedAction) {
            var time = serializedAction.time, event = serializedAction.event;
            return {
                time: time,
                event: event.$kind === "MouseEvent"
                    ? new input_1.MouseEvent(event.type, event.button, event.x, event.y)
                    : new input_1.KeyboardEvent(event.type, event.key),
            };
        });
        return new TapeData(actions);
    };
    TapeData.prototype.serialize = function () {
        return {
            $kind: "TapeData",
            actions: this.actions.map(function (action) {
                var time = action.time, event = action.event;
                return {
                    $kind: "RecordedAction",
                    time: time,
                    event: (0, input_1.isMouseEvent)(event)
                        ? {
                            $kind: "MouseEvent",
                            type: event.type,
                            button: event.button,
                            x: event.x,
                            y: event.y,
                        }
                        : {
                            $kind: "KeyboardEvent",
                            type: event.type,
                            key: event.key,
                        },
                };
            }),
        };
    };
    return TapeData;
}());
exports.TapeData = TapeData;
