"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __values = (this && this.__values) || function(o) {
    var s = typeof Symbol === "function" && Symbol.iterator, m = s && o[s], i = 0;
    if (m) return m.call(o);
    if (o && typeof o.length === "number") return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
    throw new TypeError(s ? "Object is not iterable." : "Symbol.iterator is not defined.");
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var _a, _b;
Object.defineProperty(exports, "__esModule", { value: true });
exports.TapePlayer = void 0;
var kleur_1 = __importDefault(require("kleur"));
var defer_1 = __importDefault(require("@suchipi/defer"));
var pretty_print_error_1 = require("pretty-print-error");
var input = __importStar(require("../input"));
var output = __importStar(require("../output"));
var DATA = Symbol("DATA");
var IS_PLAYING = Symbol("IS_PLAYING");
var TIMEOUTS = Symbol("TIMEOUTS");
var PLAYBACK_DEFER = Symbol("PLAYBACK_DEFER");
var TapePlayer = /** @class */ (function () {
    function TapePlayer(data) {
        this[_a] = false;
        this[_b] = null;
        this[DATA] = data;
        this[TIMEOUTS] = new Set();
    }
    Object.defineProperty(TapePlayer.prototype, "isPlaying", {
        get: function () {
            return this[IS_PLAYING];
        },
        enumerable: false,
        configurable: true
    });
    TapePlayer.prototype.play = function () {
        var e_1, _c;
        var _this = this;
        if (this[IS_PLAYING]) {
            throw new Error("Attempted to play a tape that was already playing. This is disallowed.");
        }
        var _loop_1 = function (action) {
            var event_1 = action.event, time = action.time;
            var timeout = setTimeout(function () {
                try {
                    _this[TIMEOUTS].delete(timeout);
                    if (input.isMouseEvent(event_1)) {
                        switch (event_1.type) {
                            case "click": {
                                // We intentionally don't play these back, because the down/up events will do the same thing.
                                // If we play both down/up and click events, we'll double-click for every single click.
                                // output.Mouse.click(event.button!);
                                break;
                            }
                            case "down": {
                                output.Mouse.hold(event_1.button);
                                break;
                            }
                            case "up": {
                                output.Mouse.release(event_1.button);
                                break;
                            }
                            case "move": {
                                output.Mouse.moveTo(event_1.x, event_1.y);
                                break;
                            }
                        }
                    }
                    else if (input.isKeyboardEvent(event_1)) {
                        switch (event_1.type) {
                            case "down": {
                                output.Keyboard.hold(event_1.key);
                                break;
                            }
                            case "up": {
                                output.Keyboard.release(event_1.key);
                                break;
                            }
                        }
                    }
                }
                catch (err) {
                    console.log(kleur_1.default.yellow("An error was thrown while playing back a recorded action:"));
                    console.log((0, pretty_print_error_1.formatError)(err));
                    console.log(kleur_1.default.yellow("Playback of future recorded actions will continue despite the error."));
                }
            }, time);
            this_1[TIMEOUTS].add(timeout);
        };
        var this_1 = this;
        try {
            for (var _d = __values(this[DATA].actions), _e = _d.next(); !_e.done; _e = _d.next()) {
                var action = _e.value;
                _loop_1(action);
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (_e && !_e.done && (_c = _d.return)) _c.call(_d);
            }
            finally { if (e_1) throw e_1.error; }
        }
        this[IS_PLAYING] = true;
        var defer = new defer_1.default();
        this[PLAYBACK_DEFER] = defer;
        var afterSleep = function () {
            _this[IS_PLAYING] = false;
            _this[TIMEOUTS] = new Set();
        };
        var timeToSleep = (this[DATA].length || 0) + 2;
        var sleepTimeout = setTimeout(defer.resolve, timeToSleep);
        this[TIMEOUTS].add(sleepTimeout);
        return defer.promise.then(afterSleep, afterSleep);
    };
    TapePlayer.prototype.stop = function () {
        var e_2, _c;
        if (!this[IS_PLAYING]) {
            throw new Error("Attempted to stop playing a tape that wasn't being played. This isn't allowed.");
        }
        try {
            for (var _d = __values(this[TIMEOUTS]), _e = _d.next(); !_e.done; _e = _d.next()) {
                var timeout = _e.value;
                clearTimeout(timeout);
                this[TIMEOUTS].delete(timeout);
            }
        }
        catch (e_2_1) { e_2 = { error: e_2_1 }; }
        finally {
            try {
                if (_e && !_e.done && (_c = _d.return)) _c.call(_d);
            }
            finally { if (e_2) throw e_2.error; }
        }
        var defer = this[PLAYBACK_DEFER];
        if (defer != null) {
            defer.reject(new Error("Playback cancelled by `stop` call"));
            this[PLAYBACK_DEFER] = null;
        }
        this[IS_PLAYING] = false;
    };
    return TapePlayer;
}());
exports.TapePlayer = TapePlayer;
_a = IS_PLAYING, _b = PLAYBACK_DEFER;
