"use strict";
var __values = (this && this.__values) || function(o) {
    var s = typeof Symbol === "function" && Symbol.iterator, m = s && o[s], i = 0;
    if (m) return m.call(o);
    if (o && typeof o.length === "number") return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
    throw new TypeError(s ? "Object is not iterable." : "Symbol.iterator is not defined.");
};
var _a, _b, _c;
Object.defineProperty(exports, "__esModule", { value: true });
exports.TapeRecorder = void 0;
var input_1 = require("../input");
var event_filters_1 = require("./event-filters");
var types_1 = require("../types");
var DATA = Symbol("DATA");
var RECORDING_STARTED_AT = Symbol("RECORDING_STARTED_AT");
var EVENTS_TO_IGNORE = Symbol("EVENTS_TO_IGNORE");
var IS_RECORDING = Symbol("IS_RECORDING");
var PUSH_EVENT = Symbol("PUSH_EVENT");
var LISTENERS = Symbol("LISTENERS");
var STOP_LISTENING = Symbol("STOP_LISTENING");
var TapeRecorder = /** @class */ (function () {
    function TapeRecorder(data, eventsToIgnore) {
        this[_a] = false;
        this[_b] = -1;
        this[_c] = [];
        this[DATA] = data;
        this[EVENTS_TO_IGNORE] = eventsToIgnore;
    }
    Object.defineProperty(TapeRecorder.prototype, "isRecording", {
        get: function () {
            return this[IS_RECORDING];
        },
        enumerable: false,
        configurable: true
    });
    TapeRecorder.prototype.start = function () {
        if (this[IS_RECORDING]) {
            throw new Error("Attempted to start a tape recorder that was already recording. This isn't allowed.");
        }
        this[RECORDING_STARTED_AT] = Date.now();
        var boundPushEvent = this[PUSH_EVENT].bind(this);
        this[LISTENERS] = [
            input_1.Mouse.onClick(types_1.MouseButton.ANY, boundPushEvent),
            input_1.Mouse.onDown(types_1.MouseButton.ANY, boundPushEvent),
            input_1.Mouse.onUp(types_1.MouseButton.ANY, boundPushEvent),
            input_1.Mouse.onMove(boundPushEvent),
            input_1.Keyboard.onDown(types_1.Key.ANY, boundPushEvent),
            input_1.Keyboard.onUp(types_1.Key.ANY, boundPushEvent),
        ];
        this[IS_RECORDING] = true;
    };
    TapeRecorder.prototype[(_a = IS_RECORDING, _b = RECORDING_STARTED_AT, _c = LISTENERS, PUSH_EVENT)] = function (event) {
        var e_1, _d;
        var now = Date.now();
        var shouldPush = true;
        try {
            for (var _e = __values(this[EVENTS_TO_IGNORE]), _f = _e.next(); !_f.done; _f = _e.next()) {
                var filter = _f.value;
                shouldPush = shouldPush && !(0, event_filters_1.eventMatchesFilter)(event, filter);
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (_f && !_f.done && (_d = _e.return)) _d.call(_e);
            }
            finally { if (e_1) throw e_1.error; }
        }
        if (shouldPush) {
            this[DATA].actions.push({
                event: event,
                time: now - this[RECORDING_STARTED_AT],
            });
        }
    };
    TapeRecorder.prototype[STOP_LISTENING] = function () {
        var e_2, _d;
        try {
            for (var _e = __values(this[LISTENERS]), _f = _e.next(); !_f.done; _f = _e.next()) {
                var listener = _f.value;
                listener.stop();
            }
        }
        catch (e_2_1) { e_2 = { error: e_2_1 }; }
        finally {
            try {
                if (_f && !_f.done && (_d = _e.return)) _d.call(_e);
            }
            finally { if (e_2) throw e_2.error; }
        }
        this[LISTENERS] = [];
    };
    TapeRecorder.prototype.finish = function () {
        if (!this[IS_RECORDING]) {
            throw new Error("Attempted to stop a tape recorder that was already stopped. This isn't allowed.");
        }
        this[STOP_LISTENING]();
        this[IS_RECORDING] = false;
    };
    return TapeRecorder;
}());
exports.TapeRecorder = TapeRecorder;
