"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Tape = void 0;
var tape_data_1 = require("./tape-data");
var tape_player_1 = require("./tape-player");
var tape_recorder_1 = require("./tape-recorder");
var DATA = Symbol("DATA");
var CURRENT_STATE = Symbol("CURRENT_STATE");
var EVENTS_TO_IGNORE = Symbol("EVENTS_TO_IGNORE");
var RECORDER = Symbol("RECORDER");
var PLAYER = Symbol("PLAYER");
var TapeState = {
    RECORDING: "RECORDING",
    PLAYING: "PLAYING",
    IDLE: "IDLE",
};
var Tape = /** @class */ (function () {
    function Tape(eventsToIgnore, data) {
        if (eventsToIgnore === void 0) { eventsToIgnore = []; }
        if (data === void 0) { data = new tape_data_1.TapeData([]); }
        this[_a] = TapeState.IDLE;
        this[DATA] = data;
        this[EVENTS_TO_IGNORE] = eventsToIgnore;
        this[RECORDER] = new tape_recorder_1.TapeRecorder(data, eventsToIgnore);
        this[PLAYER] = new tape_player_1.TapePlayer(data);
    }
    Object.defineProperty(Tape.prototype, "state", {
        get: function () {
            return this[CURRENT_STATE];
        },
        enumerable: false,
        configurable: true
    });
    Tape.prototype.record = function () {
        if (this[CURRENT_STATE] === TapeState.RECORDING) {
            throw new Error("Attempted to record to a tape that was already being recorded to. This isn't allowed.");
        }
        if (this[CURRENT_STATE] === TapeState.PLAYING) {
            throw new Error("Attempted to record to a tape that was being played. This isn't allowed.");
        }
        this[RECORDER].start();
        this[CURRENT_STATE] = TapeState.RECORDING;
    };
    Tape.prototype.stopRecording = function () {
        if (this[CURRENT_STATE] !== TapeState.RECORDING) {
            throw new Error("Attempted to stop recording a tape that wasn't being recorded to. This isn't allowed.");
        }
        this[RECORDER].finish();
        this[CURRENT_STATE] = TapeState.IDLE;
    };
    Tape.prototype.play = function () {
        var _this = this;
        if (this[CURRENT_STATE] === TapeState.PLAYING) {
            throw new Error("Attempted to play a tape that was already being played. This isn't allowed.");
        }
        if (this[CURRENT_STATE] === TapeState.RECORDING) {
            throw new Error("Attempted to play a tape that was being recorded to. This isn't allowed.");
        }
        var promise = this[PLAYER].play();
        this[CURRENT_STATE] = TapeState.PLAYING;
        promise.then(function () {
            _this[CURRENT_STATE] = TapeState.IDLE;
        });
        return promise;
    };
    Tape.prototype.stopPlaying = function () {
        if (this[CURRENT_STATE] !== TapeState.PLAYING) {
            throw new Error("Attempted to stop playing a tape that wasn't being played. This isn't allowed.");
        }
        this[PLAYER].stop();
        this[CURRENT_STATE] = TapeState.IDLE;
    };
    Tape.prototype.serialize = function () {
        return {
            $kind: "Tape",
            eventsToIgnore: this[EVENTS_TO_IGNORE],
            data: this[DATA].serialize(),
        };
    };
    Tape.deserialize = function (serialized) {
        return new Tape(serialized.eventsToIgnore, tape_data_1.TapeData.deserialize(serialized.data));
    };
    var _a;
    _a = CURRENT_STATE;
    Tape.State = TapeState;
    return Tape;
}());
exports.Tape = Tape;
